// In MS-Windows, a number is returned when a button is
// pressed, and the programmer is responsible for keeping track of
// what number was what button.  In X-Windows, each button has a pointer
// to a global function to call (a "callback") when the button is
// pressed.  In C++, a big complaint (of X) is that you can't call
// a member function.  Some toolkits will use a member function pointer
// for the base class, and then do casting to get a pointer to the
// derived class.  This method is NOT safe when multiple inheritance
// or virtual inheritance are used.

// The ``right'' solution is Actions.  No casting is needed.

// Actions utilize C++ templates to form a tiny class for each class
// that contains a member function you want to call.  For example,
// if you have an Application class with a member function Close,
// you'd have an action CallbackAction<Application> which contains
// an Application pointer as well as a member function pointer to
// Application::Close.

// Actions can also handle C global functions, C++ global functions,
// and class static member functions.

#include <srcomm3/Action.h>

Action::~Action()
{
}

GlobalAction::GlobalAction( void (*f)() )
{
    f_ = f;
}

void GlobalAction::execute()
{
	(*f_)();
}
