#include <string.h>
#include <alloc.h>
#include <cstring.h>

char *Emptystring = "\0";

static unsigned mksize(unsigned s)
{
	s++;
	s = 32*((31+s)/32) - 1;
	return s;
}

CC string::string()
{
	s = Emptystring;
	size = len = 0;
}

CC string::string( const char* a )
{
	_copy(a);
}

CC string::string( const string& x )
{
	_copy(x);
}

CC string::string( char c, int sz )
{
	size = mksize(sz);
	len = sz;
	s = new char[size+1];
	if( s )
	{
		memset( s, c, sz );
		s[sz] = '\0';
	}
}

string::~string()
{
	_kill();
}

string& CC string::_copy( const string& x )
{
	len = x.len;
	size = mksize(len);
	if( x.s == Emptystring )
	{
		s = Emptystring;
		size = len = 0;
	}
	else
	{
		s = new char[size+1];
		if( s ) memmove( s, x.s, len+1 ); else _kill();
	}
	return *this;
}

string& CC string::_copy( const char* a )
{
	if( a )
	{
		len = strlen( a );
		size = mksize(len);
	}
	else
		size = len = 0;
	if( size > 0 )
	{
		s = new char[size+1];
		if( s ) memmove( s, a, len+1 ); else _kill();
	}
	else
		s = Emptystring;
	return *this;
}

string& CC string::_kill()
{
	if( s != Emptystring && s ) delete s;
	s = Emptystring;
	len = 0;
	size = 0;
	return *this;
}

string& CC string::_resize( int newlen )
{
	if( newlen >= size-1 )
	{
		if( size < 15 ) size = 15;
		while( newlen >= size-1 )
			size = size*2;
		size = mksize( size );
		char *n = new char[ size+1 ];
		if( n ) memmove( n, s, len+1 );
		if( s != Emptystring ) delete s;
		s = n;
	}
	return *this;
}

string& CC string::operator = ( const string& x )
{
	if( &x == this ) return *this;
	_kill();
	_copy( x );
	return *this;
}

string& CC string::operator = ( const char* x )
{
	_kill();
	_copy( x );
	return *this;
}

string& CC string::operator += ( char c )
{
	if( len+1 >= size-1 )
		_resize(len+1);
	s[len++] = c;
	s[len] = 0;
	return *this;
}

string& CC string::operator += ( const string& s )
{
	insert( len, s );
	return *this;
}

string& CC string::operator += ( const char* s )
{
	insert( len, s );
	return *this;
}

string& CC string::prepend( const string& s )
{
    insert( 0, s );
    return *this;
}

string& CC string::prepend( const char* s )
{
    insert( 0, s );
    return *this;
}

char& CC string::operator () ( int i )
{
	static char dummy;
	if( (unsigned)i > len )
	{
		dummy = '\0';
		return dummy;
	}
	if( i == len )
	{
		_resize(len+1);
		s[i+1] = '\0';
	}
	return s[i];
}

char CC string::operator[] ( int index ) const
{
	return (unsigned)index >= len ? '\0' : s[index];
}

string CC string::copy( int start, int ln ) const
{
    string c;
    if( start > len )
        start = len;
    if( ln < 0 )
        ln = len;
    int end = start+ln;
    if( end > len )
        end = len;
    for( int i = start; i < end; i++ )
        c += s[i];
    return c;
}

string& CC string::remove( int start, int ln )
{
    if( ln == -1 ) ln = len;
	if( start > len ) start = len;
	if( start+ln > len )
        ln = len-start;
    if( ln > 0 )
    {
        memmove( s+start, s+start+ln, len-(start+ln)+1 );
        len -= ln;
    }
    return *this;
}

string& CC string::insert( int position, string x )
{
    if( position > len ) position = len;
	_resize( len+::length(x) );
	memmove( s+position+::length(x), s+position, len-position+1 );
	memmove( s+position, x.s, ::length(x) );
	len += ::length(x);
    return *this;
}

int CC operator == ( const string& a, const string& b )
{
	return ( a.len == b.len ) && !memcmp( a.s, b.s, a.len );
}

int CC operator != ( const string& a, const string& b )
{
	return ( a.len != b.len ) || !!memcmp( a.s, b.s, a.len );
}

int CC operator == ( const string& a, const char* b )
{
    return !strcmp( a.s, b );
}

int CC operator != ( const string& a, const char* b )
{
    return !!strcmp( a.s, b );
}

int CC operator < ( const string& a, const string& b )
{
    return strcmp( a.s, b.s ) < 0;
}

int CC operator > ( const string& a, const string& b )
{
    return strcmp( a.s, b.s ) > 0;
}

int CC operator <= ( const string& a, const string& b )
{
    return strcmp( a.s, b.s ) <= 0;
}

int CC operator >= ( const string& a, const string& b )
{
    return strcmp( a.s, b.s ) >= 0;
}

string CC operator + ( string a, string b )
{
	string c( '\0', a.len + b.len );
	memcpy( c.s, a.s, a.len );
	memcpy( c.s+a.len, b.s, b.len );
	c.len = a.len+b.len;
	c.s[c.len] = '\0';
	return c;
}


