//////////////////////////////////////////////////////////////////////
// DATETIME                                                         //
//////////////////////////////////////////////////////////////////////
// These are utility functions and a class for manipulating dates   //
// and times.                                                       //
//////////////////////////////////////////////////////////////////////

#pragma hdrstop
#include <dos.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>

#include <srcomm3/inout.h>
#include <srcomm3/datetime.h>

char Time::TimeSeparatorChar = ':';
bool Time::Hour24 = false;

Time Time::MIN(LONG_MIN);
Time Time::MAX(LONG_MAX);

static struct time unix2time( long x )
{
	struct time t;
	struct date d;
	unixtodos( x, &d, &t );
	return t;
}

static struct date unix2date( long x )
{
	struct time t;
	struct date d;
	unixtodos( x, &d, &t );
	return d;
}

Time::Time()
{
	time( &unixdate );
}

long Time::Second() const
{
	return unix2time(unixdate).ti_sec;
}

long Time::Minute() const
{
	return unix2time(unixdate).ti_min;
}

long Time::Hour() const
{
	return unix2time(unixdate).ti_hour;
}

long Time::Day() const
{
	return unix2date(unixdate).da_day;
}

long Time::Month() const
{
	return unix2date(unixdate).da_mon;
}

long Time::Year() const
{
	return unix2date(unixdate).da_year;
}

long operator - ( Time a, Time b )
{
	return a.unixdate - b.unixdate;
}

Time operator + ( Time a, long b )
{
	return Time(a.Clock()+b);
}

Time DateOnly( Time x )
{
	struct time t;
	struct date d;
	unixtodos( x.unixdate, &d, &t );
	t.ti_hund = 0;
	t.ti_sec = 0;
	t.ti_min = 0;
	t.ti_hour = 0;
	return Time( dostounix( &d, &t ) );
}

Time::Time( int year, int mon, int day )
{
	struct time t;
	struct date d;
	t.ti_hund = 0;
	t.ti_sec = 0;
	t.ti_min = 0;
	t.ti_hour = 0;
	d.da_year = year;
	d.da_mon = mon;
	d.da_day = day;
	unixdate = dostounix( &d, &t );
}

standardout& operator << ( standardout& out, Time t )
{
	return out << DateDisplay( t );
}

const char * DateDisplay( Time t )
{
    if( t == Time::MIN )    return "BigBang";
    if( t == Time::MAX )    return "End-of-Time";

	struct date dd;
	struct time tt;

	unixtodos( t.unixdate, &dd, &tt );

	static char s[40];
	s[0] = '\0';
	itoa( dd.da_day, s, 10 );
	strcat( s, "-" );

	const char* mon[] = { "", "Jan", "Feb", "Mar", "Apr", "May", "Jun",
						  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
	strcat( s, mon[dd.da_mon] );
	strcat( s, "-" );
	itoa( dd.da_year, s+strlen(s), 10 );
	if( tt.ti_hour != 0 || tt.ti_min != 0 || tt.ti_sec != 0 || tt.ti_hund != 0 )
	{
		strcat( s, " " );
		int i = tt.ti_hour;
		if( !Time::Hour24 )
		{
			if( i >= 12 ) i -= 12;
			if( i == 0 ) i = 12;
		}
		itoa( i, s+strlen(s), 10 );

		i = strlen(s);
		s[i++] = Time::TimeSeparatorChar;
		s[i++] = '0' + tt.ti_min / 10;
		s[i++] = '0' + tt.ti_min % 10;
#if 1
        s[i++] = Time::TimeSeparatorChar;
        s[i++] = '0' + tt.ti_sec / 10;
        s[i++] = '0' + tt.ti_sec % 10;
#if 0
        s[i++] = '.';
        s[i++] = '0' + tt.ti_hund / 10;
        s[i++] = '0' + tt.ti_hund % 10;
#endif
#endif
		if( !Time::Hour24 )
		{
			if( tt.ti_hour < 12 )
				s[i++] = 'a';
			else
				s[i++] = 'p';
			s[i++] = 'm';
		}
		s[i] = '\0';
	}

	return s;
}

Time Today = DateOnly(Time());

standardin& operator >> ( standardin& in, Time& t )
{
	int pos = 0;
	const char* full = "mm/dd/yyyy";
	char s[15] = "\0";

	in.begininput();
	for( const char *p = full; *p; p++ )
		in.out( *p );
	for(;;)
	{
		char ch;
		ch = in.get();
		if( ( ch == '\r' || ch == '\n' || ch == 0 ) &&
			( pos == 6 || pos == 8 ) )
		{
			int y, m, d;

			if( pos == 6 )
			{
				y = (s[4]-'0') * 10 + (s[5]-'0');
				if( y > 90 )    y += 1900;
                else        y += 2000;
			}
			else
				y = (s[4]-'0') * 1000
				  + (s[5]-'0') * 100
				  + (s[6]-'0') * 10
				  + (s[7]-'0');

			m = (s[0]-'0') * 10 + (s[1]-'0');
			d = (s[2]-'0') * 10 + (s[3]-'0');
			Time temp(y,m,d);
			if( temp.Year() != y || temp.Month() != m || temp.Day() != d )
			{
				for( int i = 0; i < 2+pos; i++ )
				{
					in.out( '\b' );
					in.out( ' ' );
					in.out( '\b' );
				}
				pos = 0;
				for( p = full; *p; p++ )
					in.out( *p );
			}
			else
			{
				in.out( '\n' );
				t = temp;
				break;
			}
		}

        if( ch == 0 )
        {
            in.out( '\n' );
            t = 0;
            break;
        }

        if( ch == '/' )
        {
            if( pos == 1 )
            {
                s[1] = s[0];
                s[0] = '0';
                in.out( '\b' );
                in.out( '0' );
                in.out( s[1] );
                in.out( '/' );
                in.out( 'd' );
                in.out( 'd' );
                pos = 2;
                continue;
            }
            if( pos == 3 )
            {
                s[3] = s[2];
                s[2] = '0';
                in.out( '\b' );
                in.out( '0' );
                in.out( s[3] );
                in.out( '/' );
                in.out( 'y' );
                in.out( 'y' );
                in.out( 'y' );
                in.out( 'y' );
                pos = 4;
                continue;
            }
        }
        if( ch >= '0' && ch <= '9' )
        {
            switch( pos )
            {
                case 0:
                    for( p = full; *p; p++ )
                    {
                        in.out( '\b' );
                        in.out( ' ' );
                        in.out( '\b' );
                    }
                    in.out( s[0] = ch );
                    pos++;
                    break;
                case 1:
                    in.out( s[1] = ch );
                    in.out( '/' );
                    in.out( 'd' );
                    in.out( 'd' );
                    pos++;
                    break;
                case 2:
                    in.out( '\b' );
                    in.out( '\b' );
                    in.out( s[2] = ch );
                    pos++;
                    break;
                case 3:
                    in.out( s[3] = ch );
                    in.out( '/' );
                    in.out( 'y' );
                    in.out( 'y' );
                    in.out( 'y' );
                    in.out( 'y' );
                    pos++;
                    break;
                case 4:
                    in.out( '\b' );
                    in.out( '\b' );
                    in.out( ' ' );
                    in.out( ' ' );
                    in.out( '\b' );
                    in.out( '\b' );
                    in.out( '\b' );
                    in.out( '\b' );
                    in.out( s[4] = ch );
                    pos++;
                    break;
                case 5:
                case 6:
                case 7:
                    in.out( s[pos] = ch );
                    pos++;
                    break;
            }
        }

        if( ch == '\b' && pos > 0 )
        {
            int num = 1;
            if( pos == 2 ) num = 4;
            if( pos == 4 ) num = 6;
            for( int i = 0; i < num; i++ )
            {
                in.out( '\b' );
                in.out( ' ' );
                in.out( '\b' );
            }
            pos--;
            if( pos == 0 )
                for( p = full; *p; p++ )
                    in.out( *p );
            if( pos == 2 )
            {
                in.out( 'd' );
                in.out( 'd' );
            }
            if( pos == 4 )
            {
                in.out( 'y' );
                in.out( 'y' );
                in.out( 'y' );
                in.out( 'y' );
            }
        }
    }
    in.endinput();
    return in;
}

