//////////////////////////////////////////////////////////////////////
// DOOR                                                             //
//////////////////////////////////////////////////////////////////////
// The DoorInfo class reads information from DOORFILE.SR and allows //
// the programmer to access it.                                     //
//////////////////////////////////////////////////////////////////////

#include <time.h>
#include <dos.h>
#include <mem.h>
#include <ctype.h>
#include <conio.h>
#include <stdlib.h>

#include <cstring.h>
#include <unit/StrUtl.h>

#include <srcomm3/ansi.h>
#include <srcomm3/fossil.h>

#include <srcomm3/types.h>
#include <srcomm3/door.h>
#include <srcomm3/srinout.h>
#include <srcomm3/finout.h>

#include <srcomm3/srparam.h>

// DOORFILE.SR includes:
//   Username
//   Ansi     ( 1 = yes, 0 = no, -1 = ask )
//   IBMChar  ( 1 = yes, 0 = no, -1 = same as ANSI )
//   PageLen  ( #, or 0 = default )
//   BaudRate ( #, or 0 = default )
//   ComPort  ( #, or 0 = Local )
//   TimeLimit( #, or -1 = no limit )
//   Real name

DoorInfo DOOR;
const char *DoorInfo::DoorFile = "DOORFILE.SR";

DoorInfo::DoorInfo()
{
    ansi = 1;
    ibmcharacters = 1;
    pagelength = 25;
    baudrate = 2400;
    comport = 0;
    timelimit = -1;
    loaded = 0;
}

void DoorInfo::init()
{
	filein f( DoorFile );

    if( !!f )
    {
		string s;
        long l;

        f >> name;
        f >> s;     ansi = int( val(s) );
        f >> s;     ibmcharacters = int( val(s) );
        f >> s;     pagelength = int( val(s) );
        f >> s;     l = val(s);
				baudrate = (unsigned)( l > 65535u ? 38400u : l );
        f >> s;     comport = int( val(s) );
        f >> s;     l = val(s);
                timelimit = (int)( l > 32767u ? 32767u : l );
        f >> realname;
    }

    if( !!f )
        loaded = 1;

	realbaudrate = baudrate;

	Resource rAnsi( "Ansi", ansi );
	Resource rIBMCharacters( "IBMCharacters", ibmcharacters );
    Resource rCheckTimeLimit( "CheckTimeLimit", 1L );
    Resource rLocalPlay( "Local", 0L );
    Resource rLockBaud( "LockBaudRate", 0L );
	Resource rTimeOut( "TimeOut", GameInput::NoKeyTimeOut );

	GameInput::NoKeyTimeOut = rTimeOut.Int();
    if( rAnsi.Long() != -1 )
		ansi = rAnsi.Boolean();
	if( rIBMCharacters.Long() != -1 )
		ibmcharacters = rIBMCharacters.Boolean();
	if( !rCheckTimeLimit.Boolean() )
		timelimit = -1;
	if( rLocalPlay.Boolean() )
		comport = 0;
	if( rLockBaud.Long() != 0 )
	{
		long l = rLockBaud.Long();

		if( l >= 300 && l <= 65535u )
        {
			baudrate = (unsigned)l;
			local << "Locking baud rate at " << attr(11) << baudrate << attr() << " baud.\n";
        }
		else
        {
            const char *s = "Invalid resource 'LockBaudRate':  baud rate must be from 300 to 65535 baud.\n";
            local << attr(14) << s << attr();
        }
	}

	GameOutput::Ansi = ansi;
	GameOutput::IBMCharacters = ibmcharacters;
}

DoorInfo::~DoorInfo()
{
}

int DoorInfo::read()
{
    if( !loaded )
    {
        const char *errormsg = " not found.  Please run the install program!\n";
        kout << DoorFile << errormsg;
        exit(1);
	}

	if( pagelength == 0 || pagelength < 10 )
		pagelength = 23;

    // Load global variables
    GameOutput::ComPort = comport;

    if( comport == 0 )
	{
		if( ansi == -1 )
			ansi = 1;
	}
	else
	{
		char c;
        Fossil::init( comport, baudrate );
        if( ansi == -1 )
		{
			kout << "Do you have ANSI color?  [Y/n] ";

			kout.SetRawMode();
			do
			{
				kin >> c;
				c = toupper( c );
				if( c == '\r' ) c = 'Y';
			} while( c != 'Y' && c != 'N' );
			kout.ResetRawMode();

			ansi = ( c == 'Y' );

			kout << c << '\n';
		}
	}

	if( ibmcharacters == -1 )
		ibmcharacters = ansi;
	else
		ibmcharacters &= 1;

	// Load global variables
	GameOutput::Ansi = ansi;
	GameOutput::IBMCharacters = ibmcharacters;
	GameOutput::ComPort = comport;

		kout.setColorTranslator( 0 );

	return 1;
}

int displayfile( string filename )
{
	filein f( filename, 1 );

	if( !f )    return 0;

	while( !!f )
	{
        string s;
		for(;;)
		{
			char c;
			f >> c;
			if( c == '\n' || !f )
				break;
			s += c;
		}
		if( !f )
			break;
		if( kin.keyhit() )
		{
			char a;
			kin >> a;
			if( a == ' ' )
			{
				kout << attr(4) << "\nAborted\n" << attr();
				return 1;
			}
		}
		kout << s << '\n';
	}

	kout << attr();
    return 1;
}

