//////////////////////////////////////////////////////////////////////
// DOORGAME                                                         //
//////////////////////////////////////////////////////////////////////
// Here is where the game and modem are set up and launched.  This  //
// unit reads the door information, checks several resource options //
// (such as ansi, modem configuration, fossil enable, vga effects), //
// initializes the other units, sets up the overlay manager if you  //
// have enabled it in the compiler options, and then runs           //
// DoorGame::PlayGame(), which you must define.                     //
//////////////////////////////////////////////////////////////////////

#include <dos.h>
#include <alloc.h>
#include <new.h>
#include <stdlib.h>

#include <srcomm3/types.h>
#include <srcomm3/longint.h>
#include <srcomm3/datetime.h>
#include <cstring.h>
#include <unit/StrUtl.h>
#include <srcomm3/srinout.h>
#include <srcomm3/News.h>
#include <Array.h>
#include <srcomm3/Door.h>
#include <srcomm3/DoorGame.h>
#include <srcomm3/srtime.h>
#include <srcomm3/async.h>
#include <srcomm3/SrParam.h>
#include <srcomm3/coninout.h>
#include <srcomm3/finout.h>
#ifndef __OS2__
#include <unit/Vga.h>
#include <unit/Mtask.h>
#endif
#include <srcomm3/DesqView.h>

#include <srcomm3/srtime.h>

#ifndef __OS2__
static Resource rVGA( "Display.VGA", 0L );
#endif

void out_of_memory();
void fileerrorhandler( string s );

Time   DoorGame::startPlayTime;
string DoorGame::name;
string DoorGame::abbrev;
string DoorGame::version;
int    DoorGame::nodeLock = 1;
StatusBar* DoorGame::statusBar = NULL;

DoorGame::DoorGame( string n, string a, string v )
{
	name = n;
	abbrev = a;
    version = v;
	nodeLock = 1;
}

void DoorGame::Shutdown()
{
#ifndef __OS2__
	if( rVGA.Boolean() )
		SetVGABorder( 0 );
#endif
}

DoorGame::~DoorGame()
{
}

void DoorGame::Run()
{
    // random number generator
    randomize();

    // Initialize the wansi unit
    void ansi_init();
    ansi_init();

    // Detect DESQview
    void _DetectStartupMultitasker();
	_DetectStartupMultitasker();

	// Filtering
	void initfilter();
	initfilter();

	// Clear the screen
    sout << attr(7) << '\n';
	sout.Clear();

    // Out of memory handling
	set_new_handler( out_of_memory );

	// Install an error handler for the file system
	FileSystem::InstallErrorHandler( fileerrorhandler );

    // Status bar
    if( !statusBar )
        statusBar = MakeStatusBar();
	{
        // Set the Pause colors
		PauseColor.bg = statusBar->Color();
		PauseColor.fg = statusBar->Color() + 8;
        PauseColor.bc = 15;
    }

	// Remove the cursor from the screen
    sout.NoCursor();

    // Install an error handler for the file system
    FileSystem::InstallErrorHandler( fileerrorhandler );

    // Door unit initialization
    DOOR.init();

    // Read all the system-related resources
	StandardSetup();

    Resource rStatusDisplay( "System.StatusBars", 1L );
	if( rStatusDisplay.Boolean() )
	{
		statusBar->Show();
		statusBar->Display( abbrev, "", '*' );
	}
	else
		statusBar->Hide();

#ifndef __OS2__
	// Set up the overlay system
	kout << aNumber << (unsigned long)coreleft() << aNormal << " bytes free";
#endif

#ifdef __OVERLAY__
    Resource rExpanded( "ExpandedSwapping", (long)0 );
    Resource rExtended( "ExtendedSwapping", (long)0 );

    // Overlay Initialization
	kout << "; " << aNumber << (16L*_ovrbuffer) << aNormal
		 << " bytes (conventional) used for swap space.\n";
	if( !rExtended.Boolean() || _OvrInitExt( 0, 0 ) )
		if( !rExpanded.Boolean() || _OvrInitEms( 0, 0, 0 ) )
			kout << "Disk swapping\n";
		else
			kout << "EMS swapping\n";
	else
		kout << "Extended memory swapping\n";
#else
	kout << ".\n";
#endif

	void InstallKeyHooks();
	InstallKeyHooks();

	if( nodeLock && SetLockout() )
	{
		SetupModem();
		DOOR.read();
		kout << "Sorry, it appears that another user is playing this game of "
			 << name << " at this time.\n";
		exit(2);
	}

#ifdef MTASK
	new ConTask( 0 );
	if( rStatusDisplay.Boolean() )
	{
		new ConTask( DisplayTimeTask );
		new ConTask( UpdateStatusTask );
	}
	GameInput::InstallHook( swap );
#endif

	if( !LocalParams() )
		return;

	SetupModem();

	DOOR.read();
	if( DOOR.ComPort() == 0 )
		GameOutput::PageLength--;
	if( DOOR.ComPort() != 0 && DOOR.BaudRate() <= 1200 )
		GameOutput::SpecialEffects = 0;
	if( DOOR.Ansi() )
		remote.setColorTranslator( AnsiTranslator );

	Resource rBlankLocalScreen( "System.BlankLocalScreen", 0L );
	if( rBlankLocalScreen.Boolean() && DOOR.ComPort() != 0 )
	{
		kout.LocalOutput = 0;
			local << "Someone is playing " << name << "!\n";
	}

	if( kout.GetLogFileOutput() )
	{
		extern fileout LogFile;

		GameOutput::LogFilePrefix = " ";
		LogFile << attr(8)
				<< "\n"
				<< "\n";
		LogFile << '' << attr(15) << " Log File for '"
				<< DOOR << "':  \n" << aNormal;
	}

	PlayGame();
}

StatusBar* DoorGame::MakeStatusBar()
{
    return new StatusBar;
}

bool DoorGame::LocalParams()
{
    return true;
}

void DoorGame::SetupModem()
{
	for( int port = 1; port <= 8; port++ )
	{
		string s = "COM" + str(port);
		Resource rDefineCom( s.c_str(), "" );

		if( rDefineCom.length() == 2 )
		{
			long base = rDefineCom.Long(0);
			long irq = rDefineCom.Long(1);
            if( irq != 0 && base != 0 )
                Async::set_comport_parameters( port, int(base), int(irq) );
		}
	}
}

void DoorGame::StandardSetup()
{
	Resource rSpecialEffects( "SpecialEffects", GameOutput::SpecialEffects );
	Resource rDisplay( "Display.Video", "Direct" );
	Resource rMultitasker( "System.Multitasker", 0L );
	Resource rFossil( "System.Fossil", 0L );
	Resource rBIOS( "BIOS", 0L );

	extern int try_fossil;
	try_fossil = rFossil.Boolean();

	GameOutput::SpecialEffects = rSpecialEffects.Boolean();
	string display = rDisplay;
	if( eq( display, "Direct" ) )
		conout::DirectVideo();
	if( eq( display, "BIOS" ) )
		conout::BIOSVideo();
	if( rBIOS.Boolean() )
		conout::BIOSVideo();

	// Country-dependent information
	{
		COUNTRY c;
		country( 0, &c );
		if( c.co_tmsep[0] )
			Time::TimeSeparatorChar = c.co_tmsep[0];
		if( c.co_thsep[0] )
			standardout::SetCommaMode( c.co_thsep[0] );
	}

	// Does the sysop want commas in the output?
	Resource rCommaFormat( "Output.Comma", 1L );
	if( !rCommaFormat.Boolean() )
	{
		string comma = rCommaFormat;
		if( length(comma) == 1 )
			standardout::SetCommaMode( comma[0] );
		else
			standardout::ResetCommaMode();
	}

	// What is the time separator character?
	Resource rTimeSeparator( "Output.Time.Separator", "" );
	{
		string sep = rTimeSeparator;
		if( length(sep) == 1 )
			Time::TimeSeparatorChar = sep[0];
	}

	// AM/PM format?
	Resource rTime24Hour( "Output.Time.24Hour", 0L );
	Time::Hour24 = rTime24Hour.Boolean();

	// Multitasking system
	if( !rMultitasker.Boolean() )
		DisableMultitasker();

#ifndef __OS2__
	if( rVGA.Boolean() )
	{
		int r[16] = { 0, 6,10,28,35,27,40,46,21,32,34,34,54,51,63,63 };
		int g[16] = { 0, 6,24,36, 9, 9,29,46,21,33,51,53,36,32,58,63 };
		int b[16] = { 0,30,13,42, 9,27,17,43,21,51,34,53,36,53,42,63 };

		for( int i = 0; i < 16; i++ )
			SetRGBPalette( i, 4*r[i], 4*g[i], 4*b[i] );

		if( !statusBar )
			statusBar = MakeStatusBar();
		Resource rBorder( "Display.Border", statusBar->Color() );
		SetVGABorder( rBorder.Int() );
	}
#endif

	Resource rLog( "Game.Log", "" );
	if( length( rLog ) > 0 )
        kout.OpenLogFile( rLog );

	atexit( DoorGame::Shutdown );
}

void DoorGame::ReleaseLockout()
{
    unlink( "InUse.FLG" );
}

int DoorGame::SetLockout()
{
    int OK = 0;
    Time now;
    {
        filein f( "InUse.FLG" );
        if( !!f )
        {
            string username;
			f >> username;
			f.setw();
			if( eq( username, DOOR ) )
				OK = 1;

			long lastused;
			f >> lastused;
			if( now.Clock() - lastused > 30 * MINUTES )
				// half an hour
				OK = 1;
		}
		else
			// no file
			OK = 1;
	}

	if( !OK ) return 1;

    fileout f( "InUse.FLG", 0 );     // 0 = Overwrite
	f << DOOR << '\n';
	f << now.Clock() << '\n';
	f << "\nThis file tells " << abbrev << " that this user is still playing " << abbrev << '.';
	f << "\nIf you delete this file, another user will be able to play.";
	f << "\nThis is to prevent multiple nodes from playing simultaneously.\n";

	atexit( DoorGame::ReleaseLockout );

	return 0;
}

int myabort()
{
    // Throw an exception if you have BC++ 4.0
	abort();
	return 0;
}

static void out_of_memory()
{
	kout << attr(14) << "Out Of Memory!  Aborting...\n";
    ErrorLog << DoorGame::startPlayTime << ":  OUT OF MEMORY!!!\n";
	exit(1);
}

static void fileerrorhandler( string s )
{
	ErrorLog << Time() << ":  " << s;
}

