//////////////////////////////////////////////////////////////////////
// FOSSIL                                                           //
//////////////////////////////////////////////////////////////////////
// The Fossil class provides modem I/O; if the Fossil driver is not //
// there, it will pass calls on down to the ASYNC unit.             //
//////////////////////////////////////////////////////////////////////

#include <dos.h>

#include <srcomm3/async.h>
#include <srcomm3/fossil.h>

int try_fossil = 0;
static union REGS fossil_r;

static unsigned char AL;
static unsigned int  DX;
static unsigned char COM = 0;

unsigned char Fossil::exists = 0;

static void int14call( unsigned char AH )
{
    if( try_fossil )
    {
        fossil_r.h.al = AL;
        fossil_r.h.ah = AH;
        fossil_r.x.dx = DX;
        int86( 0x14, &fossil_r, &fossil_r );
        AL = fossil_r.h.al;
        DX = fossil_r.x.dx;
    }
}

int Fossil::init( unsigned comport, unsigned baudrate )
{
    if( exists || comport == 0 )
		return 1;

	COM = comport;

    if( try_fossil )
    {
        DX = COM-1;
        // fossil_r.x.bx = 0x4F50; to get ^C flag byte
        fossil_r.x.bx = 0;
        int14call( 4 );
    }

    if( !try_fossil || fossil_r.x.ax != 0x1954 )
    {
        // FOSSIL does not exist
        Async::init();
        if( baudrate == 0 ) baudrate = 2400;
        return Async::open( comport, baudrate );
    }

    if( try_fossil )
    {
        exists = 1;

        AL = 0;
        if( baudrate >= 300 )   AL = 2;
        if( baudrate >= 600 )   AL = 3;
        if( baudrate >= 1200 )  AL = 4;
        if( baudrate >= 2400 )  AL = 5;
        if( baudrate >= 4800 )  AL = 6;
        if( baudrate >= 9600 )  AL = 7;
        if( baudrate >= 19200 ) AL = 0;
        if( baudrate >= 38400u )AL = 1;
        AL <<= 5;
        AL |= 3;
        DX = COM-1;
        int14call( 0 );

        return 1;
    }

    return 0;
}

int Fossil::deinit()
{
	if( !exists )
	{
        if( COM != 0 ) Async::close();
		return 1;
	}

    DX = COM-1;
	int14call( 5 );

	return 1;
}

char Fossil::write( char c )
{
    if( !COM )    return 0;
    if( !exists )   return Async::write( c );

	DX = COM-1;
	AL = c;
	int14call( 0x01 );

    return 0;
}

char Fossil::read()
{
    if( !COM )    return 0;
    if( !exists )   return Async::read();

    DX = COM-1;
    AL = 0;
    int14call( 0x02 );
    return fossil_r.h.al;
}

int Fossil::buffer()
{
    if( !COM )    return 0;
    if( !exists )   return Async::buffer_check();

    DX = COM-1;
    int14call( 0x03 );
    return ( fossil_r.h.ah & 0x01 ) != 0;
}

int Fossil::carrierdetect()
{
    if( !COM )    return 1;
    if( !exists )   return Async::carrierdetect();

	DX = COM-1;
    int14call( 0x03 );
    return ( AL & 0x80 ) != 0;
}

void Fossil::hangup()
{
    if( !COM )    return;
	if( !exists )
	{
        Async::hangup();
		deinit();
		return;
    }

    deinit();
    return;     // NOT SURE HOW TO DO IN FOSSIL!!!
}

void __fossil_exit()
{
    Fossil::deinit();
}
#pragma exit __fossil_exit

