#ifndef SRCOMM3_Inout_h
#define SRCOMM3_Inout_h

#ifndef CC
#define CC
#endif

#include <srcomm3/types.h>
#include <cstring.h>
#ifndef _cstring_h
inline int length( const string& s ) { return s.length(); }
#endif

class standardout;
class LineBuffer
{
  private:
	char *buf;
    int *fg;
    int *bg;
	int bufpos;
	int linewidth;
	int bufsize;

	void copy( const LineBuffer& );
	void kill();

  public:
	LineBuffer( int w );
	LineBuffer( const LineBuffer& b );
	~LineBuffer();

	LineBuffer& operator = ( const LineBuffer& );

    void CC insert( char c, int f, int b );
    void CC narrow( int w );
    void CC backspace();

	void clear();
	char operator [] ( int index );
	int fore( int index );
	int back( int index );

	int pos() { return bufpos; }
	int width() { return linewidth; }

	friend standardout& operator << ( standardout& out, const LineBuffer& b );
};

class standardin
{
  private:
	int fieldwidth;
    long num_default;
    long num_max;

  public:
	standardin();

    standardin& CC setdefault( long def = -1 );
    standardin& CC setmax( long max = -1 );

    int CC width();
    standardin& CC setw( int w = 0 );

	virtual char CC out( char c );
	void CC outs( const char * s );
	virtual int CC keyhit() { return 0; }
    virtual char CC get() { return 0; }
    virtual void CC begininput();
    virtual void CC endinput();
	virtual void CC getstr( string& s );
    virtual long CC readnum( long max );

	standardin& CC operator >> ( char& c );
	standardin& CC operator >> ( long& l );
	standardin& CC operator >> ( string& s );
	standardin& CC operator >> ( int& i );
	standardin& CC operator >> ( unsigned& i );
	standardin& CC operator >> ( unsigned char& i );
};

typedef void (*OutputTranslator)( standardout& out, const string& s );
typedef void (*ColorTranslator)( standardout& out );
void AnsiTranslator( standardout& out );
void StandardOutputTranslator( standardout& out, const string& s );

class standardout
{
  private:
    LineBuffer buffer;
    static char CommaCharacter;

  protected:
    int rawmode;
	shortint current_fore, current_back;
	shortint new_fore, new_back;

    OutputTranslator translate;
    ColorTranslator changeColorFunc;

  public:
	void writenum( long );

    standardout( int newwidth = 0 );
    ~standardout();

    LineBuffer& CC narrow( int w )  { buffer.narrow( w ); return buffer; }
    LineBuffer CC Buffer()          { return buffer; }

    void CC SetRawMode()            { rawmode = 1; }
    void CC ResetRawMode()          { rawmode = 0; }
    int CC RawMode()                { return rawmode; }

    void CC SetActualAttr( int f, int b = -1 ) { current_fore = f; current_back = b==-1? current_back : b; }
	virtual void CC SetAttr( int f, int b = -1 );
    int CC GetActualForeground() { return current_fore; }
    int CC GetActualBackground() { return current_back; }
    int CC GetForeground() { return new_fore; }
	int CC GetBackground() { return new_back; }

    void changecolor();
    ColorTranslator setColorTranslator( ColorTranslator t );
    OutputTranslator setOutputTranslator( OutputTranslator t );

    static void CC SetCommaMode( char c = ',' )     { CommaCharacter = c; }
    static void CC ResetCommaMode()                 { CommaCharacter = 0; }
    static char CC CommaMode()                      { return CommaCharacter; }

    virtual void CC put( int len, const char c[] );
    virtual void CC _put( int len, const char c[] );

    standardout& CC operator << ( const char *s );
	standardout& CC operator << ( const string& s );
	standardout& CC operator << ( char c );
	standardout& CC operator << ( unsigned char i );
    standardout& CC operator << ( int i )           { writenum(i); return *this; }
    standardout& CC operator << ( unsigned i )      { writenum(i); return *this; }
    standardout& CC operator << ( long l )          { writenum(l); return *this; }
    standardout& CC operator << ( unsigned long l ) { writenum(l); return *this; }
};

// Color object
struct attr
{
	// foreground and background data
	int fore;
	int back;

    // Default color is gray
	//   If b is -1, then the previous background is used (unchanged)
    CC attr( int f = 7, int b = -1 );

	// Output turns the attr into an ANSI sequence
	friend standardout& CC operator << ( standardout& out, attr& a );
};

#endif
