//////////////////////////////////////////////////////////////////////
// RECFILE                                                          //
//////////////////////////////////////////////////////////////////////
// This unit gives you record files, which are random-access files  //
// of fixed size structs.  It's very easy to save and load structs, //
// but beware that pointers are not saved properly.  It just does a //
// byte-by-byte transfer to and from disk.                          //
//////////////////////////////////////////////////////////////////////

#include <cstring.h>
#include <srcomm3/finout.h>
#include <srcomm3/recfile.h>
#include <srcomm3/FileMgr.h>

#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <process.h>
#include <share.h>
#include <errno.h>
#include <dos.h>
#include <stdlib.h>

recordfile::recordfile( string filename, int s )
{
    lastaccess = -1;
    err = 0;
	size = s;
	handle = -1;
    open( filename );
}

recordfile::recordfile( int s )
{
    lastaccess = -1;
    err = 0;
	size = s;
	handle = -1;
}

recordfile::~recordfile()
{
    close();
}

int recordfile::open( string filename )
{
	if( exists() )
		close();

    filename = FileManager::Translate( filename );

    handle = ::open( filename.c_str(),
			 O_RDWR | O_CREAT | O_BINARY | SH_DENYNONE,
			 S_IREAD | S_IWRITE );

	if( handle == -1 )
	{
		err = errno;

		if( err == ENOENT )
			FileSystem::HandleError( "File '%s' not openable.\n", filename );
		if( err == EMFILE )
			FileSystem::HandleError( "Too many files open.  Try increasing FILES= in CONFIG.SYS.\n", "" );
		if( err == EACCES )
			; // Probably a LOCK on the file
	}

    pathname = filename;

	return 0;
}

int recordfile::close()
{
	if( !exists() )   return 0;

    ::close( handle );
    handle = -1;
	err = 0;

    pathname = "";

    return 0;
}

int CC recordfile::read( long n, void *data, int count )
{
	if( !exists() )  return 0;

    if( n >= filesize() ) lastaccess = -1;
    long l = (n==lastaccess)? 0 : ::lseek( handle, size * n, SEEK_SET );
    lastaccess = n+count;

    if( l == -1 )
	{
		err = errno;
        lastaccess = -1;
        return 0;
	}

	int status = _read( handle, data, count * size );
	if( status == -1 )
	{
        err = errno;
        lastaccess = -1;
        return 0;
    }

	if( status < count * size )
	{
		err = 1;
		lastaccess = -1;
		return 0;
	}

	return 1;
}

int CC recordfile::write( long n, void *data, int count )
{
	if( !exists() )  return 0;

    if( n >= filesize() ) lastaccess = -1;
    long l = (n==lastaccess)? 0 : ::lseek( handle, size * n, SEEK_SET );
	lastaccess = n+count;

    if( l == -1 )
    {
        err = errno;
		lastaccess = -1;
		return 0;
    }

    int status = _write( handle, data, count * size );

	if( status == -1 )
	{
        lastaccess = -1;
		err = errno;
		return 0;
    }

	if( status < count * size )
	{
        lastaccess = -1;
		FileSystem::HandleError( "Error writing to file '%s', disk full?\n", pathname );

        err = 1;
		return 0;
    }

    return 1;
}

int recordfile::lock( long n )
{
    err = ::lock( handle, size * n, size );

	return ( err == 0 );
}

int recordfile::unlock( long n )
{
    err = ::unlock( handle, size * n, size );

    return ( err == 0 );
}

long CC recordfile::filesize()
{
    if( !exists() )  return 0;

	long l = ::filelength( handle );
    if( l == -1 )
	{
        err = errno;
        return 0;
	}

    return ( l / size );
}

int CC recordfile::eraseFile()
{
	string s = pathname;

    close();
	int status = unlink( s.c_str() );
    open( s );
	return status != -1;
}

int CC recordfile::error()
{
	int t = err;

    err = 0;
	return t;
}

