//////////////////////////////////////////////////////////////////////
// SRMENU                                                           //
//////////////////////////////////////////////////////////////////////
// This unit gives you menus like SRE's, including an interface to  //
// the resource file.  A lot of this needs to be moved out to a     //
// string formatting unit...                                        //
//////////////////////////////////////////////////////////////////////

#include <string.h>

#include <ctype.h>
#include <stdlib.h>

#include <srcomm3/types.h>
#include <srcomm3/srinout.h>
#include <Array.h>
#include <cstring.h>
#include <unit/StrUtl.h>
#include <unit/DynArray.h>
#include <srcomm3/srparam.h>
#include <srcomm3/srmenu.h>

char Menu::KeyBracketL = '[';
char Menu::KeyBracketR = ']';
char Menu::TitleBracketL = '[';
char Menu::TitleBracketR = ']';
char Menu::DefaultBracketL = '[';
char Menu::DefaultBracketR = ']';

// From SRFILTER.CPP
int filterlength( const string& s );

char Menu::SecretKey = '@';
char (*Menu::SecretHandler)() = 0;

int Menu::Waiting = 0;
int Menu::Interrupted = 0;

static string spaces( int w )
{
    return string( ' ', w );
}

static string FormatLeft( const string& s, int w )
{
	int len = filterlength( s );
    return s + spaces( w-len );
}

static string FormatRight( const string& s, int w )
{
	int len = filterlength( s );
    return spaces(w-len) + s;
}

static string FormatCenter( const string& s, int w )
{
	int len = filterlength( s );
    int wL = (w-len)/2;
    int wR = (w-len) - wL;
    return spaces(wL) + s + spaces(wR);
}

void error()
{
    kout << "Error in MENU - please look at SYSOP.LOG for details.\n";
    exit( 1 );
}

void parseString( string s, DynArray<string>& p )
{
	while( length(s) > 0 )
    {
        while( s[0] && isspace( s[0] ) )
            s.remove( 0, 1 );
        string t;
        while( s[0] && !isspace( s[0] ) )
        {
            t += s[0];
            s.remove( 0, 1 );
		}
		if( length(t) > 0 )
            p.append( t );
	}
}

void parseList( string s, DynArray<string>& p )
{
    while( s[0] == '(' || isspace( s[0] ) )
        s.remove( 0, 1 );
	while( s[length(s)-1] == ')' || isspace( s[length(s)-1] ) )
		s.remove( length(s)-1, 1 );
    parseString( s, p );
}

Menu::Menu( string _name, string _title, byte _tc, byte _lc, byte _bc, byte _kc, byte _dc,
	byte _c1, byte _c2, byte _c3, byte _c4, byte _c5 ):
    name( _name ), title( _title )
{
	maxcols = 1;
	interruptcode = -1;
	interrupthandler = 0;

	prompt = 0;
	header = 0;

    string res = "Menu." + name + ".Title";
	Resource T( res.c_str(), "%%%" );
	if( T != string("%%%") )
		title = T;

	int cols[10] = { _tc, _lc, _bc, _kc, _dc, _c1, _c2, _c3, _c4, _c5 };
    string resource = "Menu." + name + ".Colors";
    Resource C( resource.c_str(), "" );
    DynArray<string> parsed;
    parseList( C, parsed );
	for( int i = 0; i < parsed.length() && i < 5; i++ )
    {
        string s = parsed[i];
        if( s != "-" )
			cols[i] = val(s);
    }
    tc = cols[0];
    lc = cols[1];
	bc = cols[2];
    kc = cols[3];
	for( i = 0; i < 6; i++ )
		c[i] = cols[4+i];
}

Menu::~Menu()
{
	delete prompt;
	delete header;
	delete interrupthandler;
}

int Menu::AddItem( int code, char k, string s, MenuItem::Position p )
{
	{
		string resource = "Menu." + name + "." + str( code ) + ".Name";
		Resource N( resource.c_str(), "%%%" );
		if( N != string("%%%") )
			s = N;
	}

	MenuItem M( name, s, k, code, p );
	item.append( M );
	item[item.length()-1].SetColors( bc, kc, c[0], c[1], c[2], c[3], c[4], c[5] );
	return item.length()-1;
}

MenuItem* Menu::Item( int i )
{
	for( int j = 0; j < item.length(); j++ )
        if( item[j].code == i )
            return &item[j];
	return 0;
}

int Menu::Go( bool showmenu, int def )
{
    char c;

	{
        string res = "Menu." + name + ".Columns";
        Resource T( res.c_str(), 0L );
		if( T.Long() > 0 )
			maxcols = int(T.Long());
    }

    if( showmenu )
        Display();
	else
    {
        kout << '\n' << attr(lc%16,lc/16) << "" << TitleBracketL << attr(tc%16,tc/16);
		kout << title;
        kout << attr(lc%16,lc/16) << TitleBracketR << "" << aNormal << '\n';
    }

    if( DisplayPrompt() && def != -1 )
    {
		for( int i = 0; i < item.length(); i++ )
            if( item[i].code == def && item[i].key > ' ' )
            {
                if( kc != 0 )
					kout << aNormal << DefaultBracketL << "Enter="
						 << attr(kc%16,kc/16) << item[i].key
						 << aNormal << DefaultBracketR << "  "
                         << aCursor;
                break;
            }
    }

    do
    {
		int secretive = 0;
		Interrupted = 0;
		Waiting = 1;
		kin >> c;
		Waiting = 0;
		if( Interrupted )
		{
			if( interruptcode != -1 )
			{
				kout << aNormal << '\n';
				return interruptcode;
			}
			if( interrupthandler )
			{
				interrupthandler->execute();
				continue;
			}
			continue;
		}

		c = toupper( c );

		if( c == SecretKey && SecretHandler )
		{
			c = SecretHandler();
			if( c == 0 )
				continue;
			if( c == 1 )
			{
				DisplayPrompt();
				continue;
			}
			secretive = 1;
		}

		if( c == '?' )
		{
			if( !secretive ) kout << "Display Menu\n";
			Display();
			DisplayPrompt();
			continue;
		}

		for( int i = 0; i < item.length(); i++ )
			if( item[i].key == c )
			{
				if( !secretive )
                {
					if( length( item[i].paramv[0] ) > 0 )
						kout << item[i].paramv[0];
					else
						kout << c;
					kout << aNormal << '\n';
				}
				else
					kout << aNormal;
				return item[i].code;
			}

		if( c == 'Q' || c == '0' )  c = 27;
		if( c == '\r' )
		{
			if( def != -1 )
			{
				MenuItem* p = Item(def);
				if( p && length( p -> paramv[0] ) > 0 )
					kout << (p->paramv[0]);
            }

			kout << aNormal << '\n';
			return def;
		}

        if( c == 27 )
        {
            if( !secretive ) kout << "Quit" << aNormal << '\n';
        }
    }
    while( c != 27 );

    return -1;
}

void Menu::Display()
{
	int showkeys = 0;
	Array<int,0,MenuItem::NumParams> widths;

	for( int i = 0; i <= MenuItem::NumParams; i++ )
		widths[i] = 0;

	// Determine the length of the line that spans across the menu
	for( i = 0; i < item.length(); i++ )
	{
		Array<int,0,MenuItem::NumParams> w;

		if( item[i].key != 0 )
			showkeys = 1;

		for( int j = 0; j <= MenuItem::NumParams; j++ )
		{
			w[j] = item[i].paramwidth(j);
//			if( w[j] > 0 ) w[j] = w[j] + 1;
			if( w[j] > widths[j] ) widths[j] = w[j];
		}
	}

	int colw = 0;
	for( i = 0; i < item.length(); i++ )
	{
		int width = item[i].width( showkeys, widths );
		if( width > colw )
			colw = width;
	}

	int numitems = 0;
	int* valid = new int[item.length()];
	for( i = 0; i < item.length(); i++ )
		if( length( item[i].paramv[0] ) > 0 )
			valid[numitems++] = i;

	int cols = 77 / (colw+2);
	if( cols < 1 ) cols = 1;
	if( cols > maxcols ) cols = maxcols;
	if( cols > numitems ) cols = numitems;

	int ll = filterlength( title ) + 5;
	if( title[0] == '\\' ) ll--;
	if( cols * (colw+2) - 2 > ll )
		ll = cols * (colw+2) - 2;

	// Leave a blank line
	kout << '\n';

	// Display the title of the menu
	if( filterlength(title) > 0 )
	{
		int l = filterlength(title)+2;
		if( title[0] == '\\' ) l--;
		kout << attr(lc%16,lc/16);
		for( int i = 0; i < (ll-l)/2; i++ )
			kout << '';
		kout << TitleBracketL << attr(tc%16,tc/16);
		kout << title;
		kout << attr(lc%16,lc/16) << TitleBracketR;
		for( ; i < (ll-l); i++ )
			kout << '';
		kout << '\n';
	}
	else
	{
		kout << attr(lc%16,lc/16);
		for( int i = 0; i < ll; i++ )
			kout << '';
		kout << '\n';
	}

	// Display the header line
	if( header )
		header -> execute();

	// Display the menu line and items
	if( cols > 0 )
	{
		int numrows = ( numitems + cols - 1 ) / cols;
		for( int j = 0; j < numrows; j++ )
		{
			if( !kin.keyhit() )
				for( int i = j; i < numitems; i += numrows )
				{
					int lastcol = ( i + numrows ) >= numitems;
					item[valid[i]].display( lastcol, showkeys, widths );
					if( !lastcol ) kout << "  ";
				}
		}
	}
	if( !kin.keyhit() )
	{
		kout << attr(lc%16,lc/16);
		for( int i = 0; i < ll; i++ )
			kout << '';
		kout << '\n';
	}

	kout << aNormal;
	delete[] valid;
}

void Menu::StandardHeader()
{
}

static int __promptshown = 0;

void Menu::StandardPrompt()
{
    kout << aNormal << "Which one?  ";
    __promptshown = 1;
}

int Menu::DisplayPrompt()
{
    // Display the prompt
    __promptshown = 0;
	if( prompt )
		prompt -> execute();
	else
		StandardPrompt();
    kout << attr(15);
    return __promptshown;
}

MenuItem::MenuItem()
{
	error();
}

MenuItem::MenuItem( string n, string d, char k, int cd, Position p )
	: name(n)
{
	key = k;
	code = cd;

	paramv[0] = d;
	paramp[0] = p;
	paramc = 0;

	SetColors( 7, 7, 7, 7, 7, 7 );
}

MenuItem::~MenuItem()
{
}

int MenuItem::AddParam( const char* s, Position p )
{
	return AddParam( string(s), p );
}

int MenuItem::AddParam( const string& s, Position p )
{
	 if( paramc < NumParams )
	 {
		paramc++;
		paramv[paramc] = s;
		paramp[paramc] = p;
		return 1;
	 }
	 return 0;
}

int MenuItem::AddParam( long n, Position p )
{
	return AddParam( strcomma(n), p );
}

void MenuItem::SetColors( byte b, byte k, byte x0, byte x1, byte x2, byte x3, byte x4, byte x5 )
{
	bc = b;
	kc = k;
	c[0] = x0;
	c[1] = x1;
	c[2] = x2;
	c[3] = x3;
	c[4] = x4;
	c[5] = x5;
}

void MenuItem::updatecolors()
{
	int cols[8] = { bc, kc, c[0], c[1], c[2], c[3], c[4], c[5] };
    string resource = "Menu." + name + "." + str(code) + ".Colors";
	Resource C( resource.c_str(), "" );
    DynArray<string> parsed;
    parseList( C, parsed );
	for( int i = 0; i < parsed.length() && i < 8; i++ )
    {
        string s = parsed[i];
        if( s != "-" )
            cols[i] = val(s);
    }
	bc = cols[0];
    kc = cols[1];
	for( i = 0; i < 6; i++ )
        c[i] = cols[2+i];
}

void MenuItem::display( int lastcol, int showkeys, Array<int,0,MenuItem::NumParams>& w )
{
	if( showkeys )
	{
		if( key != 0 )
		{
			kout << attr(bc%16,bc/16) << Menu::KeyBracketL;
			kout << attr(kc%16,kc/16) << key;
			kout << attr(bc%16,bc/16) << Menu::KeyBracketR;
		}
		else
			kout << "   ";
	}

	for( int n = 0; n <= paramc; n++ )
	{
		int w1 = w[n];
		kout << attr( c[n]%16, c[n]/16 );
		if( n > 0 || showkeys )
			kout << ' ';

        if( paramp[n] == LEFT )
            kout << FormatLeft( paramv[n], w1 );
        else if( paramp[n] == RIGHT )
            kout << FormatRight( paramv[n], w1 );
        else
            kout << FormatCenter( paramv[n], w1 );
    }

    if( lastcol ) kout << '\n';
}

int MenuItem::width( int showkeys, Array<int,0,MenuItem::NumParams>& w )
{
    int w1 = showkeys? 3 : 0;
	for( int i = 0; i <= paramc; i++ )
		if( w[i] > 0 )
		{
			w1 += w[i];
			if( i > 0 || showkeys )	// space between columns
				w1++;
		}
	return w1;
}

int MenuItem::paramwidth( int p )
{
    if( p <= paramc )
		return filterlength( paramv[p] );
    else
        return 0;
}
