//////////////////////////////////////////////////////////////////////
// SRPARAM                                                          //
//////////////////////////////////////////////////////////////////////
// The main use of this unit is to provide Resource.Dat reading     //
// capabilities.  For more information on the Resource.Dat format,  //
// see SRE's documentation (RESOURCE.DOC).                          //
//                                                                  //
// You can extract values as numbers, booleans, or strings.  New in //
// version 3 of SRCOMM is the ability to have multiple values for a //
// resource.  You can extract the other values by passing in an     //
// optional parameter to the various extraction routines.           //
//////////////////////////////////////////////////////////////////////

#include <string.h>
#include <ctype.h>
#include <dos.h>
#include <bios.h>
#include <stdlib.h>

#include <srcomm3/finout.h>
#include <srcomm3/srparam.h>

#include <cstring.h>
#include <unit/StrUtl.h>

// If we're using BC++ 4, _USERENTRY is defined in _defs.h
#include <_defs.h>
#ifdef _USERENTRY
inline char getswitchar() { return '/'; }
#endif

static char Node[4] = "\0\0\0\0";
Atom ReadAtom( string& s );
//////////////////////////////////////////////////////////////////////////

Atom::Atom(): type( tNone ) { n = 0; s = ""; }

struct RList
{
    string name;
	DynArray<Atom> value;

	RList* next;
};

static RList* head = 0;

Resource::Resource( const char *s, string def )
{
	name = s;
	initialized = 0;

	set( def );
}

Resource::Resource( const char *s, const char * def )
{
	name = s;
	initialized = 0;

	set( def );
}

Resource::Resource( const char *s, long def )
{
	name = s;

	initialized = 0;

    string a = str(def);
	set( a );
}

Resource::~Resource()
{
}

long CC Resource::Long( int i )
{
	read();
	if( value.length() > i )
		return value[i].n;
	else
		return 0;
}

int CC Resource::Int( int i )
{
	long L = Long( i );
	return int(L);
}

string CC Resource::operator[] ( int i )
{
	read();
	if( i >= 0 && i < value.length() )
		return value[i].s;
	else
		return "";
}

int CC Resource::length()
{
	read();
	return value.length();
}

CC Resource::operator string()
{
	read();
    if( value.length() > 0 )
        return value[0].s;
    else
        return "";
}

void CC Resource::read()
{
	Read();
	RList* r = head;

	if( initialized ) return;
	for( int i = 1; i < _argc; i++ )
	{
		int val = -1;
		char arg[256], *p = _argv[i];

		if( *p != getswitchar() && *p != '/' && *p != '-' )
			continue;

		strcpy( arg, _argv[i]+1 );
		for( p = arg; *p && *p != '='; p++ )
			;
		if( *p )
		{
			*p = 0;
			p++;
		}

		if( eq( name, arg ) )
		{
			val = 1;
			if( *p == '0' || toupper( *p ) == 'F' || toupper( *p ) == 'N' )
				val = 0;
		}

		if( !strnicmp( arg, "NO", 2 ) && eq( name, arg+2 ) )
			val = 0;

		if( val != -1 )
		{
			arg[0] = val + '0';
			arg[1] = 0;
			set( arg );
			initialized = 1;
		}
	}

	if( initialized ) return;
	while( r )
	{
		if( eq( r->name, name ) )
		{
			set( r->value );
			initialized = 1;
		}
		r = r->next;
	}
}

void CC Resource::set( string newvalue )
{
    Atom a;
    a.s = newvalue;
    a.n = val(newvalue);
    a.type = Atom::tString;
    if( a.n != 0 )
        a.type = Atom::tNumber;
    set( a );
}

void CC Resource::set( const Atom& newvalue )
{
    if( value.length() > 0 )
		value[0] = newvalue;
	else
		value.append( newvalue );
}

void CC Resource::set( const DynArray<Atom>& newvalue )
{
	for( int i = 0; i < newvalue.length(); i++ )
		if( value.length() > i )
			value[i] = newvalue[i];
		else
			value.append( newvalue[i] );
}

void Resource::Read()
{
    void SetNode();
    static int loaded = 0;
	if( loaded ) return;
    SetNode();
	loaded = 1;

    if( Node[0] )
    {
        string filename = "RESOURCE.";
        filename = filename + Node;
        Read( filename );
    }
	Read( "RESOURCE.DAT" );
	Read( "RESOURCE.DEF" );
}

void Resource::Read( string name )
{
	filein f( name );
    string s;

	if( !f ) return;

	for(;;)
	{
		f >> s;
		if( !f )	break;

		// "#EOF" stops processing
		if( eq( s, "#EOF" ) )
			break;

		if( s[0] == ';' || s[0] == '#' || s[0] == '%' )
			continue;

        Atom o0 = ReadAtom( s );

		if( o0.type != Atom::tString )
			continue;

		RList* r = new RList;
		r -> name = o0.s;
		for(;;)
		{
			Atom o1 = ReadAtom( s );
			if( ::length(o1.s) == 0 )
				break;
			r -> value . append( o1 );
        }
        r -> next = head;
		head = r;
	}
}

int CC Resource::Boolean( int i )
{
	read();

	if( i >= value.length() )
        return 0;

	if( value[i].type == Atom::tNumber && value[i].n != 0 )
		return 1;

	if( eq( value[i].s, "true" ) ||
	  eq( value[i].s, "t" ) ||
	  eq( value[i].s, "on" ) ||
	  eq( value[i].s, "yes" ) )
		return 1;

	return 0;
}

//////////////////////////////////////////////////////////////////////////

int CC Parameter( const char *s )
{
	for( int i = 1; i < _argc; i++ )
	{
		char *p = _argv[i];

		if( *p == getswitchar() || *p == '/' || *p == '-' )
			continue;

		if( !stricmp( _argv[i], s ) )
			return 1;
	}
	return 0;
}

void SetNode()
{
    for( int i = 1; i < _argc; i++ )
    {
        char *p = _argv[i];

		if( *p == getswitchar() || *p == '/' || *p == '-' )
        {
            if( ( _argv[i][1] == 'n' || _argv[i][1] == 'N' )
               && _argv[i][2] )
            {
				Node[0] = _argv[i][2];
                Node[1] = _argv[i][3];
                Node[2] = _argv[i][4];
            }
        }
    }
}

//////////////////////////////////////////////////////////////////////////

void ReadSymbol( string& s, int& i, Atom& a )
{
	while( s[i] && isspace( s[i] ) )
		i++;

	while( s[i] && s[i] > ' ' && (unsigned char)s[i] < 255
      && s[i] != '(' && s[i] != ')'
      && s[i] != '[' && s[i] != ']'
      && s[i] != '{' && s[i] != '}'
      && s[i] != '\'' && s[i] != '"' )
        a.s += s[i++];

	a.type = Atom::tString;
}

void ReadLispObject( string& s, int& i, Atom& a )
{
	while( s[i] && isspace( s[i] ) )
		i++;

	if( s[i] == '"' )
	{
		i++;
		while( s[i] && s[i] != '"' )
		{
            if( s[i] == '~' )
			{
                i++; if( s[i] == 'n' ) a.s += '\n';
                else if( s[i] == 'b' ) a.s += '\b';
                else if( s[i] == 't' ) a.s += '\t';
                else if( s[i] == 'e' ) a.s += (char)27;
                else if( s[i] == 'a' ) a.s += '\a';
                else if( s[i] == 'v' ) a.s += '\v';
                else if( s[i] == 'r' ) a.s += '\r';
                else a.s += s[i];
			}
			else
                a.s += s[i];
			i++;
		}

		if( s[i] == '"' ) i++;
        else
        {
			a.type = Atom::tNone;
            return;
        }

        a.type = Atom::tString;
        return;
	}

	if( s[i] == '(' || s[i] == '[' || s[i] == '{' )
		i++;

	if( s[i] == ')' || s[i] == ']' || s[i] == '}' )
		i++;

	ReadSymbol( s, i, a );
	if( !isdigit( a.s[0] ) && a.s[0] != '+' && a.s[0] != '-' )
		return;
	if( ( a.s[0] == '+' || a.s[0] == '-' ) && !a.s[1] )
		return;
	for( int j = 1; j < length(a.s); j++ )
		if( !isdigit( a.s[j] ) )
			return;

	a.n = val( a.s );
	a.type = Atom::tNumber;
}

Atom ReadAtom( string& s )
{
    int i = 0;
    Atom a;
    ReadLispObject( s, i, a );
    s.remove( 0, i );
    while( length(s) > 0 && isspace( s[0] ) )
        s.remove( 0, 1 );
    return a;
}

