#ifndef SRCOMM_Types_H
#define SRCOMM_Types_H

#include <srcomm3/datetime.h>

// These are the number of seconds in one of these units of time.
// Since SRCOMM uses a longint storing the number of seconds as
// the `standard' date/time representation, you can add and subtract
// higher units of time with these constants.
const long DAYS = 86400L;
const long HOURS = 3600L;
const long MINUTES = 60L;
const long SECONDS = 1L;
const long WEEKS = (DAYS*7L);
const long YEARS = (DAYS*365L);

// This is roughly equivalent to the ANSI C++ bool type.  Since no
// existing compilers (as of June 1994) support the C++ bool type,
// I am declaring my own, and making it easy to switch to the standard.
// (The standard uses `bool', not `BOOL', `boolean', or `BOOLEAN'.)
typedef int bool;
const bool true = 1;
const bool false = 0;

// This is junk used in SRE.  SRCOMM still has remnants... :(
typedef int ID;
const ID NoID = -1;

// These are useful types
typedef unsigned char byte;
typedef unsigned short word;
typedef signed char shortint;

// Some languages define `self' as a reference to the current object.
// It seems more natural than a pointer, like C++ has, but I think
// they hadn't invented references in early versions of C++, so they
// used a pointer instead.
#define self (*this)

// Useful min and max functions
#ifndef __MINMAX_DEFINED
template <class A> A min( A a, A b ) { return a<b ? a : b; }
template <class A> A max( A a, A b ) { return a>b ? a : b; }
#endif

#endif  TYPES_H
